/*
 * Copyright (c) 1998, 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */
/*
 * Declarations for -loskit_startup functions, a random collection
 * of functions to start things up in vaguely canonical ways.
 */

#ifndef _OSKIT_STARTUP_H_
#define _OSKIT_STARTUP_H_

#include <oskit/error.h>
#include <oskit/io/blkio.h>
#include <oskit/fs/dir.h>
#include <oskit/dev/osenv.h>

/*
 * Start the system clock and plug it into libc.
 * This function should be called only once.
 */
void start_clock(void);

/*
 * Convenience function to initialize/probe device drivers,
 * using link-time dependencies to select sets to initialize.
 * This can safely be called more than once, but is not thread-safe.
 * You do not necessarily have to call this, since other start_*
 * functions do.
 */
void start_devices(void);

/*
 * More convenience functions to start either network or block devices.
 */
void start_net_devices(void);
void start_blk_devices(void);

/*
 * Open the named disk device and partition on it (or the whole disk
 * if `partition' is null).
 * This calls start_devices if need be, and causes it to probe block devices.
 */
oskit_error_t start_disk(const char *disk, const char *partition,
			 int read_only, oskit_blkio_t **out_bio);


/*
 * Initialize the disk-based filesystem code, mount a filesystem from
 * the given block device, and plug it into libc.
 */
void start_fs_on_blkio(oskit_blkio_t *part);

/* version used with pthreads */
void start_fs_on_blkio_pthreads(oskit_blkio_t *part);

/*
 * Start up the memfs code, and load the bmods from the kernel into
 * it.
 */
oskit_dir_t *start_bmod(void);

/* version used with pthreads. Returns a wrapped memfs adaptor. */
oskit_dir_t *start_bmod_pthreads(void);

/*
 * Same as above, but initialize the posix level filesystem as well.
 */
void start_fs_bmod(void);

/* version used with pthreads */
void start_fs_bmod_pthreads(void);

/*
 * Convenience code used by start_fs_bmod to fill up a memory
 * filesystem from a bmod.
 *
 * If free_routine is non-null, then it is called on each bmod
 * with free_routine(free_data, addr, len)
 *
 * Incidentally, you can pass lmm_add_free() in directly, since it
 * takes these arguments.
 */

int
bmod_populate(oskit_dir_t *dest,
	      void (*free_routine)(void *data, oskit_addr_t addr,
				   oskit_size_t len),
	      void *free_data);

/*
 * This just calls start_disk and start_fs_on_blkio for you.
 */
void start_fs(const char *diskname, const char *partname);

/* version used with pthreads */
void start_fs_pthreads(const char *diskname, const char *partname);


/*
 * Start up a network interface using BOOTP, initialize the network stack,
 * and register its socketfactory.
 * This calls start_devices if need be, and causes it to probe net devices.
 * This should be called only once.  This only configures a single network
 * device.
 *
 * Network device paramters, address and mask, can be overridden via
 * start_<name>_addr and start_<name>_mask environment variables, respectively.
 * (<name> is the interface name: usually 'de0').
 * Network host parameters (hostname, gateway, domainname and nameservers)
 * can be configured via the start_hostname, start_gateway, start_domainname
 * and start_nameservers environment variables.
 * If BOOTP information is available, that information is used as the default.
 * If no BOOTP and no environment information is supplied, the kernel will
 * prompt on the console for the information.
 */
void start_network(void);

/* version used with pthreads */
void start_network_pthreads(void);

/*
 * start_network() replacement which starts up all of the network
 * devices it finds, and tweaks a magic FreeBSD sysctl variable to
 * allow routing.
 */
void start_network_router(void);
void start_network_router_pthreads(void);

/*
 * XXX Put the following (start_conf_*) in a separate header file?
 */
#include <oskit/dev/ethernet.h>
#include <oskit/net/socket.h>
#include <oskit/net/freebsd.h>

/*
 * Initialize the network stack.  
 * Invoked by start_conf_network()
 */
int
start_conf_network_init(oskit_osenv_t *osenv,
			oskit_socket_factory_t **fsc);

/*
 * Set various host-level network paramters.  
 * Invoked by start_conf_network().
 */
int
start_conf_network_host(const char *hostname,
			const char *gateway,
			const char *domainname,
			char **nameservers);

/*
 * Open a particular ethernet interface.
 * Invoked by start_conf_network().
 */
int
start_conf_network_open_eif(oskit_etherdev_t *etherdev,
			    oskit_freebsd_net_ether_if_t **eif);
/*
 * Close a particular ethernet interface.
 * Invoked by the libstartup network shutdown code.
 */
void
start_conf_network_close_eif(oskit_freebsd_net_ether_if_t *eif);

/*
 * Provide a name for a given ethernet interface.
 * Invoked by start_conf_network().
 */
const char *
start_conf_network_eifname(int i);

/*
 * Configure a particular ethernet interface.
 * Invoked by start_conf_network().
 */
int
start_conf_network_eifconfig(oskit_freebsd_net_ether_if_t *eif,
			     const char *eif_name,
			     const char *eif_addr,
			     const char *eif_mask);

/*
 * Configure ethernet devices and networking stack.
 */
void start_conf_network(int maxdev);
void start_conf_network_pthreads(int maxdev);



/*
 * Start the SVM page-protection system, paging to the given disk partition.
 * If disk is `null', does no disk paging.  Otherwise, args are passed
 * to `start_disk' (see above).
 */
oskit_error_t start_svm(const char *disk, const char *partition);

/* version used with pthreads */
oskit_error_t start_svm_pthreads(const char *disk, const char *partition);

/*
 * Start the SVM system on a specific bio instance. In the pthreads case,
 * the bio should NOT be wrapped; it will be done internally.
 */
oskit_error_t start_svm_on_blkio(oskit_blkio_t *bio);

/* version used with pthreads */
oskit_error_t start_svm_on_blkio_pthreads(oskit_blkio_t *bio);

/*
 * Convenience function to start up clock, disk filesystem, swap, and network
 * in canonical ways and plug them into libc.
 *
 * Looks for environment/command-line variables called "root" and "swap"
 * for device/partition selection with user-friendly syntax, and variable
 * "read-only" to open the root fs read-only.  If there is no "root"
 * variable, makes the bmod fs the root fs.  If there is no "swap" variable,
 * turns on page-protection with SVM, but no disk paging.
 * Starts the network with bootp.  (See start_network() for more network
 * configuration.)
 *
 * This can safely be called more than once, but is not thread-safe.
 */
void start_world(void);

/* version used with pthreads */
void start_world_pthreads(void);

/*
 * Start the console. Typically called to replace the default console
 * stream with a real TTY device.
 */
void start_console(void);
void start_console_pthreads(void);

/*
 * Another convenience function to start the pthread system.
 */
void start_pthreads(void);

/*
 * And another convenience function to create the default osenv object.
 */
oskit_osenv_t *start_osenv(void);

/*
 * A couple of startup functions that use the unix emulation stuff
 * instead of the oskit drivers. 
 */
void	start_network_native(void);
void	start_network_native_pthreads(void);
void	start_fs_native(char *rootname);
void	start_fs_native_pthreads(char *rootname);

/*
 * Start gprof support code.
 */
void	start_gprof(void);

/*
 * atexit support for the startup library to ensure that the startup atexits
 * get run all at once, instead of interspersed with non-startup atexits.
 */
int	startup_atexit(void (*function)(void *arg), void *arg);
void	startup_atexit_init(void);

#endif /* _OSKIT_STARTUP_H_ */
